@@ -93,7 +93,7 @@ class Bundler(object):
             self._read_file(self._base_dir, self._api_filename)
 
         self._resolve_x_include()
-        self._resolve_x_pattern("x-field-pattern")
+        self._resolve_x_field_pattern()
         self._resolve_x_constraint()
         self._resolve_x_status()
         self._remove_x_include()
@@ -102,8 +102,10 @@ class Bundler(object):
         self._resolve_license()
         self._resolve_x_enum(self._content)
         self._generate_version_api_spec(self._content)
+        self._validate_integer_type()
         self._validate_field_uid()
         self._validate_response_uid()
+        self._validate_stream_type()
         self._validate_errors()
         self._validate_required_responses()
         self._resolve_strings(self._content)
@@ -366,33 +368,69 @@ class Bundler(object):
                     )
 
     def _validate_required_responses(self):
-        """Ensure all paths include a 400 and 500 response.
-
-        Print every path that does not include a 400 or 500 response.
+        """Ensure all paths include a 200 and default response.
 
         Returns
         -------
-        Exception: one or more paths is missing a 400 or 500 response
-        None: all paths have a 400 and 500 response
+        Exception: one or more paths is missing a 200 or default response
+        None: all paths have a 200 and default response
         """
         responses = self._get_parser("$..paths..responses").find(self._content)
-        required_error_codes = ["400", "500"]
+        required_responses = ["200", "default"]
         missing_paths = ""
         for response in responses:
-            missing = set(required_error_codes).difference(
-                set(response.value.keys())
-            )
-            if len(missing):
+            response_keys = [str(key) for key in response.value.keys()]
+            missing = set(required_responses).difference(set(response_keys))
+            if len(missing) > 0:
                 error_message = "{}: is missing the following required responses: {}".format(
                     response.full_path,
                     missing,
                 )
-                print(error_message)
                 missing_paths += "{}\n".format(error_message)
+
+            # TODO: need to check wheather every default schema points to
+
         if len(missing_paths) > 0:
             raise Exception(missing_paths)
+
+        # There must be the Error structure in the yaml which should have required fields code and errors
+        err_schema = self._get_parser("$..Error").find(self._content)
+        if len(err_schema) == 0:
+            raise Exception("Error schema does not exsist")
+        elif len(err_schema) > 1:
+            raise Exception(
+                "There must be exactly one instance of Error schema"
+            )
+
+        required_err_nodes = ["code", "errors"]
+        schema = err_schema[0]
+
+        if "required" not in schema.value.keys():
+            raise Exception(
+                "Error schema in %s must have the required field in it"
+                % schema.full_path,
+            )
+
+        diff = set(required_err_nodes).difference(
+            set(schema.value["required"])
+        )
+        if len(diff):
+            raise Exception(
+                "Error schema must have %s as required properties"
+                % str(required_err_nodes)
+            )
         return None
 
+    def _validate_stream_type(self):
+        for item in self._get_parser("$..x-stream").find(self._content):
+            if item.value not in ["server", "client"]:
+                parent = jsonpath_ng.Parent().find(item)[0]
+                parent_schema = parent.value
+                self._errors.append(
+                    "invalid x-stream value %s present in %s valid values are server and client"
+                    % (item.value, parent_schema["operationId"])
+                )
+
     def _validate_file(self):
         print("validating {}...".format(self._output_filename))
         with open(self._output_filename) as fid:
@@ -400,6 +438,32 @@ class Bundler(object):
             openapi_spec_validator.validate_v3_spec(yobject)
         print("validating complete")
 
+    def _validate_integer_type(self):
+        """Find all instances type in the openapi content
+        and check for allowed format
+        """
+        import jsonpath_ng
+
+        for type in self._get_parser("$..type").find(self._content):
+            valid_formats = ["int64", "int32", "uint32", "uint64"]
+            if type.value == "integer":
+                parent = jsonpath_ng.Parent().find(type)[0]
+                parent_schema = parent.value
+                if "format" not in parent_schema:
+                    print(
+                        "[WARNING]: %s has no format specified"
+                        % parent.full_path
+                    )
+                elif parent_schema["format"] not in valid_formats:
+                    self._errors.append(
+                        "%s has type integer of unsporrted format %s, supported formats are %s"
+                        % (
+                            parent.full_path,
+                            parent_schema["format"],
+                            str(valid_formats),
+                        )
+                    )
+
     def _read_file(self, base_dir, filename):
         filename = os.path.join(base_dir, filename)
         filename = os.path.abspath(os.path.normpath(filename))
@@ -429,7 +493,6 @@ class Bundler(object):
         self._resolve_refs(base_dir, yobject)
 
     def _check_upper_case(self, value):
-
         for c in value:
             if c.isupper():
                 return True
@@ -566,7 +629,7 @@ class Bundler(object):
             if (
                 len(intersect_keys) > 0
                 and "format" in value.keys()
-                and value["format"] in ["ipv4", "ipv6", "mac"]
+                and value["format"] in ["ipv4", "ipv6", "mac", "oid"]
             ):
                 stacks = inspect.stack()
                 property = "{}/{}/{}".format(
@@ -579,10 +642,10 @@ class Bundler(object):
                     stacks[1].frame.f_locals["key"],
                 )
                 self._errors.append(
-                    "Property {property} should not contain {keys} with format {format}".format(
+                    "Property {property} should not contain {keys} with format {fmt}".format(
                         property=property,
                         keys=intersect_keys,
-                        format=value["format"],
+                        fmt=value["format"],
                     )
                 )
 
@@ -610,7 +673,64 @@ class Bundler(object):
                                 )
                             )
 
-    def _resolve_x_pattern(self, pattern_extension):
+    def _validate_x_field_pattern(self, xpattern_path):
+        xpattern = xpattern_path.value
+
+        if "format" not in xpattern:
+            self._errors.append(
+                "{} is not valid, format is mandatory for x-field-pattern".format(
+                    str(xpattern_path.full_path)
+                )
+            )
+        elif xpattern["format"] == "integer" and "length" not in xpattern:
+            self._errors.append(
+                "{} property using x-field-pattern with format integer must contain length property".format(
+                    str(xpattern_path.full_path)
+                )
+            )
+        elif xpattern["format"] == "integer" and xpattern["length"] > 64:
+            self._errors.append(
+                "{} property using x-field-pattern with format integer cannot have length greater than 64".format(
+                    str(xpattern_path.full_path)
+                )
+            )
+        if "signed" in xpattern:
+            if xpattern["format"] != "integer":
+                self._errors.append(
+                    "signed property can only be used if the format is set to integer in property {}".format(
+                        str(xpattern_path.full_path)
+                    )
+                )
+            if not isinstance(xpattern["signed"], bool):
+                self._errors.append(
+                    "invalid value {} in {}, signed property can either be true or false".format(
+                        str(xpattern["signed"]), str(xpattern_path.full_path)
+                    )
+                )
+        valid_formats = ["integer", "ipv4", "ipv6", "mac", "checksum", "oid"]
+        if xpattern["format"] not in valid_formats:
+            self._errors.append(
+                "%s has unspported format %s , valid formats are %s"
+                % (
+                    str(xpattern_path.full_path),
+                    xpattern["format"],
+                    str(valid_formats),
+                )
+            )
+        valid_features = ["count", "auto", "metric_tags", "random"]
+        if "features" in xpattern:
+            for feature in xpattern["features"]:
+                if feature not in valid_features:
+                    self._errors.append(
+                        "%s has unspported feature %s , valid features are %s"
+                        % (
+                            str(xpattern_path.full_path),
+                            feature,
+                            str(valid_features),
+                        )
+                    )
+
+    def _resolve_x_field_pattern(self):
         """Find all instances of pattern_extension in the openapi content
         and generate a #/components/schemas/... pattern schema object that is
         specific to the property hosting the pattern extension content.
@@ -618,9 +738,13 @@ class Bundler(object):
         """
         import jsonpath_ng
 
+        pattern_extension = "x-field-pattern"
         for xpattern_path in self._get_parser(
             "$..{}".format(pattern_extension)
         ).find(self._content):
+
+            self._validate_x_field_pattern(xpattern_path)
+
             print("generating %s..." % (str(xpattern_path.full_path)))
             object_name = xpattern_path.full_path.left.left.left.right.fields[
                 0
@@ -628,6 +752,7 @@ class Bundler(object):
             property_name = xpattern_path.full_path.left.right.fields[0]
             property_schema = jsonpath_ng.Parent().find(xpattern_path)[0].value
             xpattern = xpattern_path.value
+
             schema_name = "Pattern.{}.{}".format(
                 "".join(
                     [
@@ -642,10 +767,10 @@ class Bundler(object):
                     ]
                 ),
             )
-            format = None
+            fmt = None
             type_name = xpattern["format"]
-            if type_name in ["ipv4", "ipv6", "mac", "x-enum"]:
-                format = type_name
+            if type_name in ["ipv4", "ipv6", "mac", "x-enum", "oid"]:
+                fmt = type_name
                 type_name = "string"
             description = "TBD"
             if "description" in xpattern:
@@ -659,7 +784,7 @@ class Bundler(object):
                 )
             else:
                 self._generate_value_schema(
-                    xpattern, schema_name, description, type_name, format
+                    xpattern, schema_name, description, type_name, fmt
                 )
 
             property_schema["$ref"] = "#/components/schemas/{}".format(
@@ -667,9 +792,14 @@ class Bundler(object):
             )
             del property_schema[pattern_extension]
 
+        if len(self._errors) > 0:
+            self._validate_errors()
+
     def _generate_checksum_schema(self, xpattern, schema_name, description):
         """Generate a checksum schema object"""
         auto_field = AutoFieldUid()
+        length = int(xpattern.get("length", 8))
+        fmt = "uint32" if length <= 32 else "uint64"
         schema = {
             "description": description,
             "type": "object",
@@ -697,8 +827,8 @@ class Bundler(object):
                 "custom": {
                     "description": "A custom checksum value",
                     "type": "integer",
-                    "minimum": 0,
-                    "maximum": 2 ** int(xpattern.get("length", 8)) - 1,
+                    "format": fmt,
+                    "maximum": 2**length - 1,
                     "x-field-uid": auto_field.uid,
                 },
             },
@@ -706,12 +836,17 @@ class Bundler(object):
         self._content["components"]["schemas"][schema_name] = schema
 
     def _generate_value_schema(
-        self, xpattern, schema_name, description, type_name, format
+        self, xpattern, schema_name, description, type_name, fmt
     ):
         auto_field = AutoFieldUid()
+
         xconstants = (
             xpattern["x-constants"] if "x-constants" in xpattern else None
         )
+
+        pattern_length = int(xpattern.get("length", 8))
+        pattern_fmt = "uint32" if pattern_length <= 32 else "uint64"
+
         schema = {
             "description": description,
             "type": "object",
@@ -738,52 +873,145 @@ class Bundler(object):
         }
         if xconstants is not None:
             schema["x-constants"] = copy.deepcopy(xconstants)
+
+        # we will follow the order in which the features are defined.
+        # This will help us to maintain backward compatibility.
+        # for example if features: ["auto", "count", "random"]
+        # The properties that will be added to yaml will be in this order with increasing x-field-uid
+
         if "features" in xpattern:
-            if "auto" in xpattern["features"]:
-                if "default" not in xpattern:
-                    self._errors.append(
-                        "default must be set for property {}, when auto feature is enabled".format(
-                            schema_name
-                        )
+            for idx, feature in enumerate(xpattern["features"]):
+                if feature == "auto":
+                    self._configure_auto_schema(
+                        xpattern,
+                        schema,
+                        schema_name,
+                        auto_field,
+                        type_name,
+                        fmt,
                     )
+
+                if idx == 0:
+                    # this is just done to keep backward compatibility
+                    # main problem is if auto is there then count starts from 6,
+                    # and when auto is not present its 5.
+                    # TODO: in future is back compatibility is broken for any reason,
+                    # we should take the advantage and remove this.
+                    # skip this UID as it was previously being used for metric_groups
+                    _ = auto_field.uid
+
+                if feature == "count":
+                    self._configure_count_schema(
+                        xpattern,
+                        schema,
+                        schema_name,
+                        auto_field,
+                        type_name,
+                        fmt,
+                        pattern_fmt,
+                        xconstants,
+                    )
+                elif feature == "metric_tags":
+                    self._configure_metric_tags_schema(
+                        xpattern, schema, schema_name, auto_field, pattern_fmt
+                    )
+                elif feature == "random":
+                    self._configure_random_schema(
+                        xpattern,
+                        schema,
+                        schema_name,
+                        auto_field,
+                        type_name,
+                        fmt,
+                    )
+
+        self._apply_common_x_field_pattern_properties(
+            schema["properties"]["value"],
+            xpattern,
+            fmt,
+            property_name="value",
+        )
+        self._apply_common_x_field_pattern_properties(
+            schema["properties"]["values"],
+            xpattern,
+            fmt,
+            property_name="values",
+        )
+        self._content["components"]["schemas"][schema_name] = schema
+
+    def _configure_auto_schema(
+        self, xpattern, schema, schema_name, auto_field, type_name, fmt
+    ):
+        if "auto" in xpattern:
+            # new enhance for auto hierarchy
+            auto_prop = xpattern["auto"]
+            if "$ref" not in auto_prop:
+                self._errors.append(
+                    "ref is a mandatory property in {}, when auto property is specified".format(
+                        schema_name
+                    )
+                )
+            if "default" not in auto_prop:
+                self._errors.append(
+                    "default is a mandatory property in {}, when auto property is specified".format(
+                        schema_name
+                    )
+                )
+            elif not isinstance(auto_prop["default"], bool):
+                self._errors.append(
+                    "only boolean values are allowed for default in {}".format(
+                        schema_name
+                    )
+                )
+            if "$ref" in auto_prop:
                 schema["properties"]["choice"]["x-enum"]["auto"] = {
                     "x-field-uid": 1
                 }
-                schema["properties"]["choice"]["default"] = "auto"
-                description = [
-                    "The OTG implementation can provide a system generated",
-                    "value for this property. If the OTG is unable to generate a value",
-                    "the default value must be used.",
-                ]
+                if "default" in auto_prop and auto_prop["default"] is True:
+                    schema["properties"]["choice"]["default"] = "auto"
                 schema["properties"]["auto"] = {
-                    "description": "\n".join(description),
-                    "type": copy.deepcopy(type_name),
+                    "$ref": auto_prop["$ref"],
                     "x-field-uid": auto_field.uid,
                 }
-                self._apply_common_x_field_pattern_properties(
-                    schema["properties"]["auto"],
-                    xpattern,
-                    format,
-                    property_name="auto",
+        else:
+            if "default" not in xpattern:
+                self._errors.append(
+                    "default must be set for property {}, when auto feature is enabled".format(
+                        schema_name
+                    )
                 )
-            if "metric_group" in xpattern["features"]:
-                schema["properties"]["metric_group"] = {
-                    "description": """A unique name is used to indicate to the system that the field may """
-                    """extend the metric row key and create an aggregate metric row for """
-                    """every unique value. """
-                    """To have metric group columns appear in the flow metric rows the flow """
-                    """metric request allows for the metric_group value to be specified """
-                    """as part of the request.""",
-                    "type": "string",
-                    "x-field-uid": auto_field.uid,
-                }
-        if "enums" in xpattern:
-            schema["properties"]["value"]["enum"] = copy.deepcopy(
-                xpattern["enums"]
-            )
-            schema["properties"]["values"]["items"]["enum"] = copy.deepcopy(
-                xpattern["enums"]
+            schema["properties"]["choice"]["x-enum"]["auto"] = {
+                "x-field-uid": 1
+            }
+            schema["properties"]["choice"]["default"] = "auto"
+            description = [
+                "The OTG implementation can provide a system generated",
+                "value for this property. If the OTG is unable to generate a value",
+                "the default value must be used.",
+            ]
+            schema["properties"]["auto"] = {
+                "description": "\n".join(description),
+                "type": copy.deepcopy(type_name),
+                "x-field-uid": auto_field.uid,
+            }
+            self._apply_common_x_field_pattern_properties(
+                schema["properties"]["auto"],
+                xpattern,
+                fmt,
+                property_name="auto",
             )
+
+    def _configure_count_schema(
+        self,
+        xpattern,
+        schema,
+        schema_name,
+        auto_field,
+        type_name,
+        fmt,
+        pattern_fmt,
+        xconstants,
+    ):
         if xpattern["format"] in ["integer", "ipv4", "ipv6", "mac"]:
             counter_pattern_name = "{}.Counter".format(schema_name)
             schema["properties"]["choice"]["x-enum"]["increment"] = {
@@ -813,47 +1041,158 @@ class Bundler(object):
                         "type": type_name,
                         "x-field-uid": counter_auto_field.uid,
                     },
+                    "count": {
+                        "type": "integer",
+                        "x-field-uid": counter_auto_field.uid,
+                    },
                 },
             }
-            if "features" in xpattern and "count" in xpattern["features"]:
-                counter_schema["properties"]["count"] = {
-                    "type": "integer",
-                    "default": 1,
-                    "x-field-uid": counter_auto_field.uid,
-                }
             self._apply_common_x_field_pattern_properties(
                 counter_schema["properties"]["start"],
                 xpattern,
-                format,
+                fmt,
                 property_name="start",
             )
             self._apply_common_x_field_pattern_properties(
                 counter_schema["properties"]["step"],
                 xpattern,
-                format,
+                fmt,
                 property_name="step",
             )
+            self._apply_common_x_field_pattern_properties(
+                counter_schema["properties"]["count"],
+                xpattern,
+                pattern_fmt,
+                property_name="count",
+            )
+
             if xconstants is not None:
                 counter_schema["x-constants"] = copy.deepcopy(xconstants)
+
             self._content["components"]["schemas"][
                 counter_pattern_name
             ] = counter_schema
-        self._apply_common_x_field_pattern_properties(
-            schema["properties"]["value"],
-            xpattern,
-            format,
-            property_name="value",
-        )
-        self._apply_common_x_field_pattern_properties(
-            schema["properties"]["values"],
-            xpattern,
-            format,
-            property_name="values",
-        )
-        self._content["components"]["schemas"][schema_name] = schema
+
+    def _configure_metric_tags_schema(
+        self, xpattern, schema, schema_name, auto_field, pattern_fmt
+    ):
+        metric_tags_schema_name = "{}.MetricTag".format(schema_name)
+        length = 65535
+        if xpattern["format"] in ["integer", "ipv4", "ipv6", "mac"]:
+            if "length" in xpattern:
+                length = int(xpattern["length"])
+            elif xpattern["format"] == "mac":
+                length = 48
+            elif xpattern["format"] == "ipv4":
+                length = 32
+            elif xpattern["format"] == "ipv6":
+                length = 128
+        metric_tags_auto_field = AutoFieldUid()
+        metric_tags_schema = {
+            "description": "Metric tag can be used to enable tracking portion of or all bits in a corresponding header field for metrics per each applicable value. These would appear as tagged metrics in corresponding flow metrics.",
+            "type": "object",
+            "required": ["name"],
+            "properties": {
+                "name": {
+                    "description": "Name used to identify the metrics associated with the values applicable for configured offset and length inside corresponding header field",
+                    "type": "string",
+                    "x-field-uid": metric_tags_auto_field.uid,
+                },
+                "offset": {
+                    "description": "Offset in bits relative to start of corresponding header field",
+                    "type": "integer",
+                    "format": pattern_fmt,
+                    "default": 0,
+                    "maximum": length - 1,
+                    "x-field-uid": metric_tags_auto_field.uid,
+                },
+                "length": {
+                    "description": "Number of bits to track for metrics starting from configured offset of corresponding header field",
+                    "type": "integer",
+                    "format": pattern_fmt,
+                    "default": length,
+                    "minimum": 1,
+                    "maximum": length,
+                    "x-field-uid": metric_tags_auto_field.uid,
+                },
+            },
+        }
+        schema["properties"]["metric_tags"] = {
+            "description": "One or more metric tags can be used to enable tracking portion of or all bits in a corresponding header field for metrics per each applicable value. These would appear as tagged metrics in corresponding flow metrics.",
+            "type": "array",
+            "items": {
+                "$ref": "#/components/schemas/{}".format(
+                    metric_tags_schema_name
+                )
+            },
+            "x-field-uid": auto_field.uid,
+        }
+        self._content["components"]["schemas"][
+            metric_tags_schema_name
+        ] = metric_tags_schema
+
+    def _configure_random_schema(
+        self, xpattern, schema, schema_name, auto_field, type_name, fmt
+    ):
+        if xpattern["format"] in ["integer", "ipv4", "ipv6", "mac"]:
+            random_pattern_name = "{}.Random".format(schema_name)
+            schema["properties"]["choice"]["x-enum"]["random"] = {
+                "x-field-uid": 6
+            }
+            schema["properties"]["random"] = {
+                "$ref": "#/components/schemas/{}".format(random_pattern_name)
+            }
+            schema["properties"]["random"]["x-field-uid"] = auto_field.uid
+            random_auto_field = AutoFieldUid()
+            random_schema = {
+                "description": "{} random pattern".format(xpattern["format"]),
+                "type": "object",
+                "properties": {
+                    "min": {
+                        "description": "The minimum possible value generated by the random value generator.",
+                        "type": type_name,
+                        "x-field-uid": random_auto_field.uid,
+                    },
+                    "max": {
+                        "description": "The maximum possible value generated by the random value generator.",
+                        "type": type_name,
+                        "x-field-uid": random_auto_field.uid,
+                    },
+                    "seed": {
+                        "description": "The seed value is used to initialize the random number generator to a deterministic state. If the user provides a seed value of 0, the implementation will generate a sequence of non-deterministic random values. For any other seed value, the sequence of random numbers will be generated in a deterministic manner (specific to the implementation).",
+                        "type": "integer",
+                        "default": 1,
+                        "format": "uint32",
+                        "x-field-uid": random_auto_field.uid,
+                    },
+                    "count": {
+                        "description": "The total number of values to be generated by the random value generator.",
+                        "type": "integer",
+                        "default": 1,
+                        "format": "uint32",
+                        "x-field-uid": random_auto_field.uid,
+                    },
+                },
+            }
+            self._apply_common_x_field_pattern_properties(
+                random_schema["properties"]["min"],
+                xpattern,
+                fmt,
+                property_name="min",
+            )
+            self._apply_common_x_field_pattern_properties(
+                random_schema["properties"]["max"],
+                xpattern,
+                fmt,
+                property_name="max",
+            )
+
+            self._content["components"]["schemas"][
+                random_pattern_name
+            ] = random_schema
 
     def _apply_common_x_field_pattern_properties(
-        self, schema, xpattern, format, property_name
+        self, schema, xpattern, fmt, property_name
     ):
         # type: (Dict, Dict, str, Union[Literal["start"], Literal["step"], Literal["value"], Literal["values"]])
         step_defaults = {
@@ -861,29 +1200,62 @@ class Bundler(object):
             "ipv4": "0.0.0.1",
             "ipv6": "::1",
         }
+        max_defaults = {
+            "mac": "ff:ff:ff:ff:ff:ff",
+            "ipv4": "255.255.255.255",
+            "ipv6": "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff",
+        }
         if "default" in xpattern:
             schema["default"] = xpattern["default"]
             if property_name == "step":
-                if format in step_defaults:
-                    schema["default"] = step_defaults[format]
+                if fmt in step_defaults:
+                    schema["default"] = step_defaults[fmt]
                 else:
                     schema["default"] = 1
+            elif property_name == "count" or property_name == "seed":
+                schema["default"] = 1
             elif property_name == "values":
                 schema["default"] = [schema["default"]]
-        if format is not None:
-            # TODO: fix this
-            # if property_name == "values":
-            #     schema["items"]["format"] = format
-            # else:
-            schema["format"] = format
-        if "length" in xpattern:
-            # TODO: fix this
-            # if property_name == "values":
-            #     schema["items"]["minimum"] = 0
-            #     schema["items"]["maximum"] = 2 ** int(xpattern["length"]) - 1
-            # else:
-            schema["minimum"] = 0
-            schema["maximum"] = 2 ** int(xpattern["length"]) - 1
+        if property_name == "max":
+            if fmt in max_defaults:
+                schema["default"] = max_defaults[fmt]
+
+        finalised_format = None
+        if xpattern["format"] == "integer":
+            finalised_format = "uint32"
+            if "length" in xpattern and xpattern["length"] > 32:
+                finalised_format = "uint64"
+            if xpattern.get("signed", False):
+                finalised_format = finalised_format[1:]
+        elif fmt is not None:
+            finalised_format = fmt
+
+        if finalised_format is not None:
+            if property_name == "values":
+                schema["items"]["format"] = finalised_format
+            else:
+                schema["format"] = finalised_format
+        if "length" in xpattern and int(xpattern["length"]) not in [32, 64]:
+            min_val = (
+                -(2 ** (int(xpattern["length"]) - 1))
+                if xpattern.get("signed", False)
+                else None
+            )
+            max_val = 2 ** int(xpattern["length"]) - 1
+            if xpattern.get("signed", False):
+                max_val = 2 ** (int(xpattern["length"]) - 1) - 1
+
+            if property_name == "values":
+                if min_val is not None:
+                    schema["items"]["minimum"] = min_val
+                schema["items"]["maximum"] = max_val
+            else:
+                if min_val is not None:
+                    schema["minimum"] = min_val
+                schema["maximum"] = max_val
+
+            if property_name == "max":
+                schema["default"] = max_val
 
     def _resolve_recursive_x_include(self, include_value):
         if "x-include" in include_value:
@@ -1033,7 +1405,6 @@ class Bundler(object):
         return dst
 
     def _get_schema_object(self, base_dir, schema_path):
-
         json_path = "$..'%s'" % schema_path.split("/")[-1]
         schema_object = self._get_parser(json_path).find(self._content)
         if len(schema_object) == 0:
