@@ -12,6 +12,8 @@ import grpc
 import semantic_version
 import types
 import platform
+import base64
+import re
 from google.protobuf import json_format
 import sanity_pb2_grpc as pb2_grpc
 import sanity_pb2 as pb2
@@ -28,6 +30,17 @@ if sys.version_info[0] == 3:
 
 openapi_warnings = []
 
+# instantiate the logger
+stderr_handler = logging.StreamHandler(sys.stderr)
+formatter = logging.Formatter(
+    fmt="%(asctime)s.%(msecs)03d [%(name)s] [%(levelname)s] %(message)s",
+    datefmt="%Y-%m-%d %H:%M:%S",
+)
+formatter.converter = time.gmtime
+stderr_handler.setFormatter(formatter)
+log = logging.getLogger("common")
+log.addHandler(stderr_handler)
+
 
 class Transport:
     HTTP = "http"
@@ -39,9 +52,11 @@ def api(
     transport=None,
     verify=True,
     logger=None,
-    loglevel=logging.INFO,
+    loglevel=logging.WARN,
     ext=None,
     version_check=False,
+    otel_collector=None,
+    otel_collector_transport="http",
 ):
     """Create an instance of an Api class
 
@@ -65,12 +80,29 @@ def api(
       man-in-the-middle (MitM) attacks. Setting verify to `False`
       may be useful during local development or testing.
     - logger (logging.Logger): A user defined logging.logger, if none is provided
-      then a default logger with a stdout handler will be provided
+      then a default logger with a stderr handler will be provided
     - loglevel (logging.loglevel): The logging package log level.
       The default loglevel is logging.INFO
     - ext (str): Name of an extension package
     """
     params = locals()
+
+    if logger is not None:
+        global log
+        log = logger
+    log.setLevel(loglevel)
+
+    if version_check is False:
+        log.warning("Version check is disabled")
+
+    if otel_collector is not None:
+        if sys.version_info[0] == 3 and sys.version_info[1] >= 7:
+            log.info("Telemetry feature enabled")
+        else:
+            raise Exception(
+                "Telemetry feature is only available for python version >= 3.7"
+            )
+
     transport_types = ["http", "grpc"]
     if ext is None:
         transport = "http" if transport is None else transport
@@ -81,8 +113,10 @@ def api(
                 )
             )
         if transport == "http":
+            log.info("Transport set to HTTP")
             return HttpApi(**params)
         else:
+            log.info("Transport set to GRPC")
             return GrpcApi(**params)
     try:
         if transport is not None:
@@ -105,21 +139,7 @@ class HttpTransport(object):
             else "https://localhost:443"
         )
         self.verify = kwargs["verify"] if "verify" in kwargs else False
-        self.logger = kwargs["logger"] if "logger" in kwargs else None
-        self.loglevel = (
-            kwargs["loglevel"] if "loglevel" in kwargs else logging.DEBUG
-        )
-        if self.logger is None:
-            stdout_handler = logging.StreamHandler(sys.stdout)
-            formatter = logging.Formatter(
-                fmt="%(asctime)s [%(name)s] [%(levelname)s] %(message)s",
-                datefmt="%Y-%m-%d %H:%M:%S",
-            )
-            formatter.converter = time.gmtime
-            stdout_handler.setFormatter(formatter)
-            self.logger = logging.Logger(self.__module__, level=self.loglevel)
-            self.logger.addHandler(stdout_handler)
-        self.logger.debug(
+        log.debug(
             "HttpTransport args: {}".format(
                 ", ".join(["{}={!r}".format(k, v) for k, v in kwargs.items()])
             )
@@ -131,7 +151,23 @@ class HttpTransport(object):
         self.verify = verify
         if self.verify is False:
             urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
-            self.logger.warning("Certificate verification is disabled")
+            log.warning("Certificate verification is disabled")
+
+    def _parse_response_error(self, response_code, response_text):
+        error_response = ""
+        try:
+            error_response = yaml.safe_load(response_text)
+        except Exception as _:
+            error_response = response_text
+
+        err_obj = Error()
+        try:
+            err_obj.deserialize(error_response)
+        except Exception as _:
+            err_obj.code = response_code
+            err_obj.errors = [str(error_response)]
+
+        raise Exception(err_obj)
 
     def send_recv(
         self,
@@ -140,6 +176,7 @@ class HttpTransport(object):
         payload=None,
         return_object=None,
         headers=None,
+        request_class=None,
     ):
         url = "%s%s" % (self.location, relative_url)
         data = None
@@ -149,11 +186,17 @@ class HttpTransport(object):
                 data = payload
                 headers["Content-Type"] = "application/octet-stream"
             elif isinstance(payload, (str, unicode)):
+                if request_class is not None:
+                    request_class().deserialize(payload)
                 data = payload
             elif isinstance(payload, OpenApiBase):
                 data = payload.serialize()
             else:
                 raise Exception("Type of payload provided is unknown")
+        log.debug("Request url - " + str(url))
+        log.debug("Method - " + str(method))
+        log.debug("Request headers - " + str(headers))
+        log.debug("Request payload - " + str(data))
         response = self._session.request(
             method=method,
             url=url,
@@ -163,6 +206,10 @@ class HttpTransport(object):
             # TODO: add a timeout here
             headers=headers,
         )
+        log.debug("Response status code - " + str(response.status_code))
+        log.debug("Response header - " + str(response.headers))
+        log.debug("Response content - " + str(response.content))
+        log.debug("Response text - " + str(response.text))
         if response.ok:
             if "application/json" in response.headers["content-type"]:
                 # TODO: we might want to check for utf-8 charset and decode
@@ -183,9 +230,7 @@ class HttpTransport(object):
                 # content types
                 return response
         else:
-            raise Exception(
-                response.status_code, yaml.safe_load(response.text)
-            )
+            self._parse_response_error(response.status_code, response.text)
 
 
 class OpenApiStatus:
@@ -200,6 +245,12 @@ class OpenApiStatus:
             # cls.logger.warning(cls.messages[key])
             logging.warning(cls.messages[key])
             object.__warnings__.append(cls.messages[key])
+            log.warning(
+                "["
+                + OpenApiStatus.warn.__name__
+                + "] cls.messages[key]-"
+                + cls.messages[key]
+            )
             # openapi_warnings.append(cls.messages[key])
 
     @staticmethod
@@ -214,6 +265,12 @@ class OpenApiStatus:
         if isinstance(func_or_data, types.FunctionType):
             return inner
         OpenApiStatus.warn(func_or_data)
+        log.warning(
+            "["
+            + OpenApiStatus.deprecated.__name__
+            + "] func_or_data-"
+            + func_or_data
+        )
 
     @staticmethod
     def under_review(func_or_data):
@@ -227,6 +284,12 @@ class OpenApiStatus:
         if isinstance(func_or_data, types.FunctionType):
             return inner
         OpenApiStatus.warn(func_or_data)
+        log.warning(
+            "["
+            + OpenApiStatus.under_review.__name__
+            + "] func_or_data-"
+            + func_or_data
+        )
 
 
 class OpenApiBase(object):
@@ -341,6 +404,7 @@ class OpenApiValidator(object):
                 return False
             return all([0 <= int(oct, 16) <= 255 for oct in mac.split(":")])
         except Exception:
+            log.debug("Validating MAC address - " + str(mac) + " failed ")
             return False
 
     def validate_ipv4(self, ip):
@@ -355,6 +419,7 @@ class OpenApiValidator(object):
         try:
             return all([0 <= int(oct) <= 255 for oct in ip.split(".", 3)])
         except Exception:
+            log.debug("Validating IPv4 address - " + str(ip) + " failed")
             return False
 
     def validate_ipv6(self, ip):
@@ -392,6 +457,7 @@ class OpenApiValidator(object):
                 ]
             )
         except Exception:
+            log.debug("Validating IPv6 address - " + str(ip) + " failed")
             return False
 
     def validate_hex(self, hex):
@@ -401,37 +467,51 @@ class OpenApiValidator(object):
             int(hex, 16)
             return True
         except Exception:
+            log.debug("Validating HEX value - " + str(hex) + " failed")
             return False
 
     def validate_integer(self, value, min, max, type_format=None):
         if value is None or not isinstance(value, int):
             return False
-        if type_format == "uint32" and value < 0:
-            return False
-        if type_format == "uint64" and value < 0:
-            return False
         if min is not None and value < min:
             return False
         if max is not None and value > max:
             return False
+        if type_format is not None:
+            if type_format == "uint32" and (value < 0 or value > 4294967295):
+                return False
+            elif type_format == "uint64" and (
+                value < 0 or value > 18446744073709551615
+            ):
+                return False
+            elif type_format == "int32" and (
+                value < -2147483648 or value > 2147483647
+            ):
+                return False
+            elif type_format == "int64" and (
+                value < -9223372036854775808 or value > 9223372036854775807
+            ):
+                return False
         return True
 
     def validate_float(self, value):
         return isinstance(value, (int, float))
 
-    def validate_string(self, value, min_length, max_length):
+    def validate_string(self, value, min_length, max_length, pattern):
         if value is None or not isinstance(value, (str, unicode)):
             return False
         if min_length is not None and len(value) < min_length:
             return False
         if max_length is not None and len(value) > max_length:
             return False
+        if pattern is not None and not re.match(pattern, value):
+            return False
         return True
 
     def validate_bool(self, value):
         return isinstance(value, bool)
 
-    def validate_list(self, value, itemtype, min, max, min_length, max_length):
+    def validate_list(self, value, itemtype, min, max, min_length, max_length, pattern):
         if value is None or not isinstance(value, list):
             return False
         v_obj = getattr(self, "validate_{}".format(itemtype), None)
@@ -444,20 +524,41 @@ class OpenApiValidator(object):
             if itemtype == "integer":
                 v_obj_lst.append(v_obj(item, min, max))
             elif itemtype == "string":
-                v_obj_lst.append(v_obj(item, min_length, max_length))
+                v_obj_lst.append(v_obj(item, min_length, max_length, pattern))
             else:
                 v_obj_lst.append(v_obj(item))
         return v_obj_lst
 
     def validate_binary(self, value):
-        if value is None or not isinstance(value, (str, unicode)):
+        if isinstance(value, bytes):
+            return True
+
+        if not isinstance(value, str):
             return False
-        return all(
-            [
-                True if int(bin) == 0 or int(bin) == 1 else False
-                for bin in value
-            ]
-        )
+            
+        try:
+            base64.b64decode(value, validate=True)
+            return True
+        except Exception:
+            pass
+            
+        # Fallback: validate as a string of '0's and '1's
+        if not value:  # An empty string is not a valid binary string in this context
+            return False
+            
+        return all(char in '01' for char in value)
+
+
+    def validate_oid(self, value):
+        segments = value.split(".")
+        if len(segments) < 2:
+            return False
+        for segment in segments:
+            if not segment.isnumeric():
+                return False
+            if not (0 <= int(segment) <= 4294967295):
+                return False
+        return True
 
     def types_validation(
         self,
@@ -469,6 +570,7 @@ class OpenApiValidator(object):
         max=None,
         min_length=None,
         max_length=None,
+        pattern=None,
     ):
         type_map = {
             int: "integer",
@@ -492,7 +594,7 @@ class OpenApiValidator(object):
             msg = "{} is not a valid or unsupported format".format(type_)
             raise TypeError(msg)
         if type_ == "list":
-            verdict = v_obj(value, itemtype, min, max, min_length, max_length)
+            verdict = v_obj(value, itemtype, min, max, min_length, max_length, pattern)
             if all(verdict) is True:
                 return
             err_msg = "{} \n {} are not valid".format(
@@ -517,7 +619,7 @@ class OpenApiValidator(object):
                 err_msg, value, type(value), min_max
             )
         elif type_ == "string":
-            verdict = v_obj(value, min_length, max_length)
+            verdict = v_obj(value, min_length, max_length, pattern)
             if verdict is True:
                 return
             msg = ""
@@ -525,6 +627,8 @@ class OpenApiValidator(object):
                 msg = ", expected min {}".format(min_length)
             if max_length is not None:
                 msg = msg + ", expected max {}".format(max_length)
+            if pattern is not None:
+                msg = msg + ", expected pattern '{}'".format(pattern)
             err_msg = "{} \n got {} of type {} {}".format(
                 err_msg, value, type(value), msg
             )
@@ -670,10 +774,13 @@ class OpenApiObject(OpenApiBase, OpenApiValidator):
                 "_DEFAULTS" in dir(self._properties[name])
                 and "choice" in self._properties[name]._DEFAULTS
             ):
-                getattr(
-                    self._properties[name],
-                    self._properties[name]._DEFAULTS["choice"],
-                )
+                choice_str = self._properties[name]._DEFAULTS["choice"]
+
+                if choice_str in self._properties[name]._TYPES:
+                    getattr(
+                        self._properties[name],
+                        self._properties[name]._DEFAULTS["choice"],
+                    )
         else:
             if default_value is None and name in self._DEFAULTS:
                 self._set_choice(name)
@@ -870,31 +977,48 @@ class OpenApiObject(OpenApiBase, OpenApiValidator):
         ):
             return
         if "enum" in details and property_value not in details["enum"]:
-            msg = (
-                "property {} shall be one of these"
-                " {} enum, but got {} at {}"
-            )
-            raise TypeError(
-                msg.format(
-                    property_name,
-                    details["enum"],
-                    property_value,
-                    self.__class__,
+            raise_error = False
+            if isinstance(property_value, list):
+                for value in property_value:
+                    if value not in details["enum"]:
+                        raise_error = True
+                        break
+            elif property_value not in details["enum"]:
+                raise_error = True
+
+            if raise_error is True:
+                msg = (
+                    "property {} shall be one of these"
+                    " {} enum, but got {} at {}"
+                )
+                raise TypeError(
+                    msg.format(
+                        property_name,
+                        details["enum"],
+                        property_value,
+                        self.__class__,
+                    )
                 )
-            )
         if details["type"] in common_data_types and "format" not in details:
             msg = "property {} shall be of type {} at {}".format(
                 property_name, details["type"], self.__class__
             )
+
+            itemtype = (
+                details.get("itemformat")
+                if "itemformat" in details
+                else details.get("itemtype")
+            )
             self.types_validation(
                 property_value,
                 details["type"],
                 msg,
-                details.get("itemtype"),
+                itemtype,
                 details.get("minimum"),
                 details.get("maximum"),
                 details.get("minLength"),
                 details.get("maxLength"),
+                details.get("pattern"),
             )
 
         if details["type"] not in common_data_types:
@@ -930,6 +1054,7 @@ class OpenApiObject(OpenApiBase, OpenApiValidator):
                 details.get("maximum"),
                 details.get("minLength"),
                 details.get("maxLength"),
+                details.get("pattern"),
             )
 
     def validate(self):
@@ -966,15 +1091,22 @@ class OpenApiObject(OpenApiBase, OpenApiValidator):
 
             if isinstance(property_name, OpenApiObject):
                 if "self" in self._STATUS and property_value is None:
-                    print("[WARNING]: %s" % self._STATUS["self"])
+                    print(
+                        "[WARNING]: %s" % self._STATUS["self"], file=sys.stderr
+                    )
 
                 return
 
             enum_key = "%s.%s" % (property_name, property_value)
             if property_name in self._STATUS:
-                print("[WARNING]: %s" % self._STATUS[property_name])
+                print(
+                    "[WARNING]: %s" % self._STATUS[property_name],
+                    file=sys.stderr,
+                )
             elif enum_key in self._STATUS:
-                print("[WARNING]: %s" % self._STATUS[enum_key])
+                print(
+                    "[WARNING]: %s" % self._STATUS[enum_key], file=sys.stderr
+                )
 
 
 class OpenApiIter(OpenApiBase):
@@ -1099,3 +1231,101 @@ class OpenApiIter(OpenApiBase):
         raise NotImplementedError(
             "validating an OpenApiIter object is not supported"
         )
+
+
+class Telemetry(object):
+    def __init__(self, endpoint, transport):
+        self.transport = transport
+        self.endpoint = endpoint
+        self.is_telemetry_enabled = False
+        self._tracer = None
+        self._trace_provider = None
+        self._resource = None
+        self._batch_span_processor = None
+        self._trace = None
+        self._http_exporter = None
+        self._grpc_exporter = None
+        self._http_instrumentor = None
+        self._grpc_instrumentor = None
+        self._spankind = None
+        if self.endpoint is not None:
+            self.is_telemetry_enabled = True
+            self._initiate_tracer()
+
+    def _initiate_tracer(self):
+        import warnings
+
+        warnings.filterwarnings("ignore", category=DeprecationWarning)
+        self._trace = importlib.import_module("opentelemetry.trace")
+        self._spankind = getattr(self._trace, "SpanKind")
+        self._trace_provider = importlib.import_module(
+            "opentelemetry.sdk.trace"
+        )
+        self._trace_provider = getattr(self._trace_provider, "TracerProvider")
+        self._resource = importlib.import_module("opentelemetry.sdk.resources")
+        self._resource = getattr(self._resource, "Resource")
+        self._batch_span_processor = importlib.import_module(
+            "opentelemetry.sdk.trace.export"
+        )
+        self._batch_span_processor = getattr(
+            self._batch_span_processor, "BatchSpanProcessor"
+        )
+        self._grpc_exporter = importlib.import_module(
+            "opentelemetry.exporter.otlp.proto.grpc.trace_exporter"
+        )
+        self._grpc_exporter = getattr(self._grpc_exporter, "OTLPSpanExporter")
+        self._http_exporter = importlib.import_module(
+            "opentelemetry.exporter.otlp.proto.http.trace_exporter"
+        )
+        self._http_exporter = getattr(self._http_exporter, "OTLPSpanExporter")
+
+        provider = self._trace_provider(
+            resource=self._resource.create({"service.name": "snappi"})
+        )
+        self._trace.set_tracer_provider(provider)
+        if self.transport == "http":
+            otlp_exporter = self._http_exporter(endpoint=self.endpoint)
+        else:
+            otlp_exporter = self._grpc_exporter(
+                endpoint=self.endpoint, insecure=True
+            )
+        span_processor = self._batch_span_processor(otlp_exporter)
+        provider.add_span_processor(span_processor)
+        tracer = self._trace.get_tracer(__name__)
+        self._tracer = tracer
+
+    def initiate_http_instrumentation(self):
+        if self.is_telemetry_enabled:
+            from opentelemetry.instrumentation.requests import (
+                RequestsInstrumentor,
+            )
+
+            RequestsInstrumentor().instrument()
+
+    def initiate_grpc_instrumentation(self):
+        if self.is_telemetry_enabled:
+            from opentelemetry.instrumentation.grpc import (
+                GrpcInstrumentorClient,
+            )
+
+            GrpcInstrumentorClient().instrument()
+
+    def set_span_event(self, message):
+        if self.is_telemetry_enabled:
+            current_span = self._trace.get_current_span()
+            current_span.add_event(message)
+
+    @staticmethod
+    def create_child_span(func):
+        def tracing(self, *args, **kwargs):
+            telemetry = self._telemetry
+            if telemetry.is_telemetry_enabled:
+                name = func.__name__
+                with self.tracer().start_as_current_span(
+                    name, kind=telemetry._spankind.CLIENT
+                ):
+                    return func(self, *args, **kwargs)
+            else:
+                return func(self, *args, **kwargs)
+
+        return tracing
