@@ -18,8 +18,9 @@ import requests
 import pkgutil
 import importlib
 from jsonpath_ng import parse
-from .openapiartplugin import OpenApiArtPlugin
+from .openapiartplugin import OpenApiArtPlugin, type_limits
 
+# TODO: get rid of this
 MODELS_RELEASE = "v0.3.3"
 
 
@@ -32,11 +33,15 @@ class FluentRpc(object):
         self.request_class = None
         self.request_property = None
         self.good_response_type = None  # considering 200-ok
+        self.proto_field_name = None  # introduced in error-ux changes
         self.bad_responses = []  # != 200-ok
         self.description = None
         self.http_method = None
         self.good_response_property = None
         self.x_status = None
+        self.stream_type = None
+        self.stream_operation_name = None
+        self.response_class = None
 
 
 class Generator:
@@ -96,7 +101,7 @@ class Generator:
     def _load_plugins(self):
         plugins = []
         pkg_dir = os.path.dirname(__file__)
-        for (_, name, _) in pkgutil.iter_modules([pkg_dir]):
+        for _, name, _ in pkgutil.iter_modules([pkg_dir]):
             module_name = "openapiart." + name
             importlib.import_module(module_name)
             obj = sys.modules[module_name]
@@ -172,6 +177,12 @@ class Generator:
             )
             common_content = common_content.replace(cnf_text, modify_text)
 
+            cnf_text = 'log = logging.getLogger("common")'
+            modify_text = 'log = logging.getLogger("{pkg_name}")'.format(
+                pkg_name=self._package_name
+            )
+            common_content = common_content.replace(cnf_text, modify_text)
+
             if re.search(r"def[\s+]api\(", common_content) is not None:
                 self._generated_top_level_factories.append("api")
             if self._extension_prefix is not None:
@@ -242,13 +253,18 @@ class Generator:
             print("found method %s" % method_name)
             rpc = FluentRpc()
             rpc.method = method_name
+            rpc.stream_type = operation.get("x-stream", "")
             rpc.operation_name = self._get_external_struct_name(
                 operation["operationId"].replace(".", "_")
             )
+            rpc.stream_operation_name = "stream" + rpc.operation_name
             rpc.description = self._get_description(operation)
             request = self._get_parser("$..requestBody..schema").find(
                 operation
             )
+            content = self._get_parser("$..requestBody..content").find(
+                operation
+            )
             for req in request:
                 (
                     _,
@@ -260,53 +276,42 @@ class Generator:
                     refs.append(ref)
                     rpc.request_property = property_name
                     rpc.request_class = class_name
+                elif "application/octet-stream" in content[0].value:
+                    rpc.request_property = "request_bytes"
+                    rpc.request_class = "RequestBytes"
 
             response_type = None
+            proto_name = None
             response_list = operation.get("responses")
             if response_list is None:
                 raise Exception("{} should have responses".format(method_name))
             for response_code, response_property in response_list.items():
-                if int(response_code) == 200:
+                if str(response_code) == "200":
                     rpc.good_response_property = response_property
                     schema_obj = self._get_parser("$..schema").find(
                         response_property
                     )
-                    if len(schema_obj) == 0:
-                        (
-                            response_name,
-                            _,
-                            _,
-                            ref,
-                        ) = self._get_object_property_class_names(
-                            response_property
-                        )
-                        if response_name is not None:
-                            response = self._get_parser('$.."$ref"').find(
-                                self._get_object_from_ref(ref)
-                            )
-                            if len(response) > 0:
-                                (
-                                    _,
-                                    response_type,
-                                    _,
-                                    ref,
-                                ) = self._get_object_property_class_names(
-                                    response[0].value
-                                )
-                                if ref:
-                                    refs.append(ref)
-                    else:
-                        (
-                            _,
-                            response_type,
-                            _,
-                            ref,
-                        ) = self._get_object_property_class_names(
-                            schema_obj[0].value
-                        )
-                        if ref:
-                            refs.append(ref)
+                    (
+                        response_name,
+                        response_type,
+                        class_name,
+                        ref,
+                        proto_name,
+                    ) = self._get_ref_from_response(
+                        schema_obj, response_property
+                    )
+                    rpc.response_class = class_name
+                    if ref:
+                        refs.append(ref)
                 else:
+                    schema_obj = self._get_parser("$..schema").find(
+                        response_property
+                    )
+                    (_, _, _, ref, _) = self._get_ref_from_response(
+                        schema_obj, response_property
+                    )
+                    if ref:
+                        refs.append(ref)
                     rpc.bad_responses.append(str(response_code))
 
             if response_type is None:
@@ -315,6 +320,7 @@ class Generator:
                 pass
 
             rpc.good_response_type = response_type
+            rpc.proto_field_name = proto_name
             rpc.http_method = path["method"]
             # TODO: restore behavior
             if "x-status" in path["operation"] and path["operation"][
@@ -327,6 +333,7 @@ class Generator:
             methods.append(
                 {
                     "name": method_name,
+                    "class_name": rpc.request_class,
                     "args": ["self"]
                     if len(request) == 0
                     else ["self", "payload"],
@@ -357,13 +364,14 @@ class Generator:
             if "type" not in schema_object:
                 continue
             print("found top level factory method %s" % property_name)
-            if schema_object["type"] == "array":
-                ref = schema_object["items"]["$ref"]
-                _, _, class_name, _ = self._get_object_property_class_names(
-                    ref
-                )
-                class_name = "%sIter" % class_name
-                self._top_level_schema_refs.append((ref, property_name))
+            # TODO: check if part needs to be removed or was introduced for which tes-case
+            # if schema_object["type"] == "array":
+            #     ref = schema_object["items"]["$ref"]
+            #     _, _, class_name, _ = self._get_object_property_class_names(
+            #         ref
+            #     )
+            #     class_name = "%sIter" % class_name
+            #     self._top_level_schema_refs.append((ref, property_name))
             self._top_level_schema_refs.append((ref, None))
 
             factories.append({"name": property_name, "class_name": class_name})
@@ -383,30 +391,44 @@ class Generator:
         super(GrpcApi, self).__init__(**kwargs)
         self._stub = None
         self._channel = None
+        self._cert = None
+        self._cert_domain = None
         self._request_timeout = 10
         self._keep_alive_timeout = 10 * 1000
+        self._maximum_receive_buffer_size = 4 * 1024 * 1024
+        self.enable_grpc_streaming = False
+        self._chunk_size = 4 * 1024 * 1024
         self._location = (
             kwargs["location"]
             if "location" in kwargs and kwargs["location"] is not None
             else "localhost:50051"
         )
         self._transport = kwargs["transport"] if "transport" in kwargs else None
-        self._logger = kwargs["logger"] if "logger" in kwargs else None
-        self._loglevel = kwargs["loglevel"] if "loglevel" in kwargs else logging.DEBUG
-        if self._logger is None:
-            stdout_handler = logging.StreamHandler(sys.stdout)
-            formatter = logging.Formatter(fmt="%(asctime)s [%(name)s] [%(levelname)s] %(message)s", datefmt="%Y-%m-%d %H:%M:%S")
-            formatter.converter = time.gmtime
-            stdout_handler.setFormatter(formatter)
-            self._logger = logging.Logger(self.__module__, level=self._loglevel)
-            self._logger.addHandler(stdout_handler)
-        self._logger.debug("gRPCTransport args: {}".format(", ".join(["{}={!r}".format(k, v) for k, v in kwargs.items()])))
+        log.debug("gRPCTransport args: {}".format(", ".join(["{}={!r}".format(k, v) for k, v in kwargs.items()])))
+        self._telemetry.initiate_grpc_instrumentation()
+
+    def _use_secure_connection(self, cert_path, cert_domain=None):
+        \"\"\"Accepts certificate and host_name for SSL Connection.\"\"\"
+        if cert_path is None:
+            raise Exception("path to certificate cannot be None")
+        self._cert = cert_path
+        self._cert_domain = cert_domain
 
     def _get_stub(self):
         if self._stub is None:
             CHANNEL_OPTIONS = [('grpc.enable_retries', 0),
-                               ('grpc.keepalive_timeout_ms', self._keep_alive_timeout)]
-            self._channel = grpc.insecure_channel(self._location, options=CHANNEL_OPTIONS)
+                               ('grpc.keepalive_timeout_ms', self._keep_alive_timeout),
+                               ('grpc.max_receive_message_length', self._maximum_receive_buffer_size)]
+            if self._cert is None:
+                self._channel = grpc.insecure_channel(self._location, options=CHANNEL_OPTIONS)
+            else:
+                crt = open(self._cert, "rb").read()
+                creds = grpc.ssl_channel_credentials(crt)
+                if self._cert_domain is not None:
+                    CHANNEL_OPTIONS.append(('grpc.ssl_target_name_override', self._cert_domain))
+                self._channel = grpc.secure_channel(
+                    self._location, credentials=creds, options=CHANNEL_OPTIONS
+                )
             self._stub = pb2_grpc.OpenapiStub(self._channel)
         return self._stub
 
@@ -417,8 +439,37 @@ class Generator:
             payload = payload.serialize()
         if isinstance(payload, dict):
             payload = json.dumps(payload)
+        elif isinstance(payload, (str, unicode)):
+            payload = json.dumps(yaml.safe_load(payload))
         return payload
 
+    def _raise_exception(self, grpc_error):
+        err = self.error()
+        try:
+            err.deserialize(grpc_error.details())
+        except Exception as _:
+            err.code = grpc_error.code().value[0]
+            err.errors = [grpc_error.details()]
+        raise Exception(err)
+
+    def _client_stream(self, stub, data):
+        data_chunks = []
+        for i in range(0, len(data), self._chunk_size):
+            if i+self._chunk_size > len(data):
+                chunk = data[i:len(data)]
+            else:
+                chunk = data[i:i+self._chunk_size]
+            data_chunks.append(pb2.Data(datum=chunk, chunk_size=self._chunk_size))
+        # print(chunk_list, len(chunk_list))
+        reqs = iter(data_chunks)
+        return reqs
+
+    def _server_stream(self, stub, responses):
+        data = b""
+        for response in responses:
+            data += response.datum
+        return data
+
     @property
     def request_timeout(self):
         \"\"\"duration of time in seconds to allow for the RPC.\"\"\"
@@ -436,6 +487,22 @@ class Generator:
     def keep_alive_timeout(self, timeout):
         self._keep_alive_timeout = timeout * 1000
 
+    @property
+    def chunk_size(self):
+        return self._chunk_size
+
+    @chunk_size.setter
+    def chunk_size(self, size):
+        self._chunk_size = size * 1024 * 1024
+
+    @property
+    def maximum_receive_buffer_size(self):
+        return self._maximum_receive_buffer_size
+
+    @maximum_receive_buffer_size.setter
+    def maximum_receive_buffer_size(self, size):
+        self._maximum_receive_buffer_size = size * 1024 * 1024
+
     def close(self):
         if self._channel is not None:
             self._channel.close()
@@ -459,8 +526,15 @@ class Generator:
                     key = "{}.{}".format("GrpcApi", rpc_method.method)
                     self._deprecated_properties[key] = rpc_method.x_status[1]
 
+                including_default, return_byte = self._process_good_response(
+                    rpc_method
+                )
+                self._write(1, "@Telemetry.create_child_span")
                 if rpc_method.request_class is None:
                     self._write(1, "def %s(self):" % rpc_method.method)
+                    self._write(
+                        2, 'log.info("Executing ' + rpc_method.method + '")'
+                    )
                     if status_msg != "":
                         self._write(2, "self.add_warnings('%s')" % status_msg)
                     self._write(2, "stub = self._get_stub()")
@@ -468,63 +542,120 @@ class Generator:
                         2,
                         "empty = pb2_grpc.google_dot_protobuf_dot_empty__pb2.Empty()",
                     )
+                    (
+                        line_indent,
+                        skip_response_check,
+                    ) = self._add_streaming_code(rpc_method, return_byte, 2)
                     self._write(
-                        2,
+                        line_indent,
                         "res_obj = stub.%s(empty, timeout=self._request_timeout)"
                         % rpc_method.operation_name,
                     )
                 else:
+                    only_bytes = (
+                        True
+                        if rpc_method.request_property == "request_bytes"
+                        else False
+                    )
                     self._write(
                         1, "def %s(self, payload):" % rpc_method.method
                     )
+                    self._write(
+                        2, 'log.info("Executing ' + rpc_method.method + '")'
+                    )
+
+                    if not only_bytes:
+                        self._write(
+                            2, 'log.debug("Request payload - " + str(payload))'
+                        )
+                        self._write(
+                            2,
+                            'self._telemetry.set_span_event("REQUEST: %s" % str(payload))',
+                        )
 
                     if status_msg != "":
                         self._write(2, "self.add_warnings('%s')" % status_msg)
 
-                    self._write(2, "pb_obj = json_format.Parse(")
-                    self._write(3, "self._serialize_payload(payload),")
-                    self._write(3, "pb2.%s()" % rpc_method.request_class)
-                    self._write(2, ")")
+                    if not only_bytes:
+                        self._write(2, "pb_obj = json_format.Parse(")
+                        self._write(3, "self._serialize_payload(payload),")
+                        self._write(3, "pb2.%s()" % rpc_method.request_class)
+                        self._write(2, ")")
                     if (
                         self._generate_version_api
                         and rpc_method.method != "get_version"
                     ):
                         self._write(2, "self._do_version_check_once()")
-                    "%s=pb_obj" % rpc_method.request_property
+
                     self._write(
                         2,
-                        "req_obj = pb2.{operation_name}Request({request_property}=pb_obj)".format(
+                        "req_obj = pb2.{operation_name}Request({request_property}={obj})".format(
                             operation_name=rpc_method.operation_name,
                             request_property=rpc_method.request_property,
+                            obj="payload" if only_bytes else "pb_obj",
                         ),
                     )
+                    if rpc_method.stream_type == "client":
+                        self._write(
+                            2,
+                            "pb_str = {obj}".format(
+                                obj="payload"
+                                if only_bytes
+                                else "pb_obj.SerializeToString()"
+                            ),
+                        )
                     self._write(2, "stub = self._get_stub()")
+                    self._write(2, "try:")
+                    # code to add streaming of config hook in set_config
+
+                    (
+                        line_indent,
+                        skip_response_check,
+                    ) = self._add_streaming_code(rpc_method, return_byte, 3)
                     self._write(
-                        2,
+                        line_indent,
                         "res_obj = stub.%s(req_obj, timeout=self._request_timeout)"
                         % rpc_method.operation_name,
                     )
-                including_default, return_byte = self._process_good_response(
-                    rpc_method
-                )
+                    self._write(2, "except grpc.RpcError as grpc_error:")
+                    self._write(3, "self._raise_exception(grpc_error)")
                 self._write(2, "response = json_format.MessageToDict(")
                 self._write(3, "res_obj, preserving_proto_field_name=True")
                 self._write(2, ")")
+                self._write(2, 'log.debug("Response - " + str(response))')
                 self._write(
-                    2, 'status_code_200 = response.get("status_code_200")'
+                    2,
+                    'self._telemetry.set_span_event("RESPONSE: %s" % str(response))',
                 )
-                self._write(2, "if status_code_200 is not None:")
                 if return_byte:
-                    self._write(
-                        3, "return io.BytesIO(res_obj.status_code_200)"
-                    )
+                    self._write(2, 'bytes = response.get("response_bytes")')
+                    self._write(2, "if bytes is not None:")
+                    self._write(3, "return io.BytesIO(res_obj.response_bytes)")
                 elif rpc_method.good_response_type:
-                    if including_default:
-                        self._write(3, "if len(status_code_200) == 0:")
+                    line_indent = 2
+                    if (
+                        rpc_method.stream_type == "server"
+                        and not skip_response_check
+                    ):
                         self._write(
-                            4, "status_code_200 = json_format.MessageToDict("
+                            line_indent, "if self.enable_grpc_streaming:"
+                        )
+                        line_indent += 1
+                        self._write(line_indent, "result = response")
+                        self._write(line_indent - 1, "else:")
+                    self._write(
+                        line_indent,
+                        'result = response.get("%s")'
+                        % rpc_method.proto_field_name,
+                    )
+                    self._write(2, "if result is not None:")
+
+                    if including_default:
+                        self._write(3, "if len(result) == 0:")
+                        self._write(4, "result = json_format.MessageToDict(")
+                        self._write(
+                            5, "res_obj.%s," % rpc_method.proto_field_name
                         )
-                        self._write(5, "res_obj.status_code_200,")
                         self._write(5, "preserving_proto_field_name=True,")
                         self._write(5, "including_default_value_fields=True")
                         self._write(4, ")")
@@ -533,23 +664,26 @@ class Generator:
                         "return self.%s().deserialize("
                         % rpc_method.good_response_type,
                     )
-                    self._write(4, "status_code_200")
+                    self._write(4, "result")
                     self._write(3, ")")
                 else:
-                    self._write(3, 'return response.get("status_code_200")')
-                for rsp_code in rpc_method.bad_responses:
-                    self._write(
-                        2,
-                        """if response.get("status_code_{code}") is not None:""".format(
-                            code=rsp_code
-                        ),
-                    )
-                    self._write(
-                        3,
-                        """raise Exception({code}, response.get("status_code_{code}"))""".format(
-                            code=rsp_code
-                        ),
-                    )
+                    self._write(2, 'resp_str = response.get("string")')
+                    self._write(2, "if resp_str is not None:")
+                    self._write(3, 'return response.get("string")')
+                # bad responses for now only has default which is not needed to be listed below
+                # for rsp_code in rpc_method.bad_responses:
+                #     self._write(
+                #         2,
+                #         """if response.get("status_code_{code}") is not None:""".format(
+                #             code=rsp_code
+                #         ),
+                #     )
+                #     self._write(
+                #         3,
+                #         """raise Exception({code}, response.get("status_code_{code}"))""".format(
+                #             code=rsp_code
+                #         ),
+                #     )
 
     def _process_good_response(self, rpc_method):
         including_default = False
@@ -596,6 +730,7 @@ class Generator:
             self._write(1, "def __init__(self, **kwargs):")
             self._write(2, "super(HttpApi, self).__init__(**kwargs)")
             self._write(2, "self._transport = HttpTransport(**kwargs)")
+            self._write(2, "self._telemetry.initiate_http_instrumentation()")
             self._write()
             self._write(1, "@property")
             self._write(1, "def verify(self):")
@@ -611,6 +746,7 @@ class Generator:
                 if method["x_status"] is not None:
                     key = "{}.{}".format("HttpApi", method["name"])
                     self._deprecated_properties[key] = method["x_status"][1]
+                self._write(1, "@Telemetry.create_child_span")
                 self._write(
                     1,
                     "def %s(%s):"
@@ -626,6 +762,7 @@ class Generator:
                 self._write(0)
                 self._write(2, "Return: %s" % method["response_type"])
                 self._write(2, '"""')
+                self._write(2, 'log.info("Executing ' + method["name"] + '")')
 
                 if method["x_status"] is not None:
                     status_msg = "%s api is %s, %s" % (
@@ -662,6 +799,11 @@ class Generator:
                         else "None"
                     ),
                 )
+                if (
+                    method["class_name"] is not None
+                    and method["class_name"] != "RequestBytes"
+                ):
+                    self._write(3, "request_class=%s," % method["class_name"])
                 self._write(2, ")")
 
     def _write_api_class(self, methods, factories):
@@ -696,14 +838,57 @@ class Generator:
                 self._write(2, "if self._version_check is None:")
                 self._write(3, "self._version_check = False")
                 self._write(2, "self._version_check_err = None")
+                self._write(2, "self._client_name = None")
+                self._write(2, "self._client_ver = None")
+                self._write(2, "self._server_name = None")
             else:
                 self._write(2, "pass")
+            self._write(2, 'endpoint = kwargs.get("otel_collector")')
+            self._write(
+                2, 'transport = kwargs.get("otel_collector_transport")'
+            )
+            self._write(2, "self._telemetry = Telemetry(endpoint, transport)")
+
+            self._write()
+            self._write(1, "def tracer(self):")
+            self._write(2, "return self._telemetry._tracer")
 
             self._write()
             self._write(1, "def add_warnings(self, msg):")
-            self._write(2, "print('[WARNING]: %s' % msg)")
+            self._write(2, "print('[WARNING]: %s' % msg, file=sys.stderr)")
             self._write(2, "self.__warnings__.append(msg)")
 
+            self._write()
+            self._write(1, "def _deserialize_error(self, err_string):")
+            self._write(2, "# type: (str) -> Union[Error, None]")
+            self._write(2, "err = self.error()")
+            self._write(2, "try:")
+            self._write(3, "err.deserialize(err_string)")
+            self._write(2, "except Exception:")
+            self._write(3, "err = None")
+            self._write(2, "return err")
+
+            self._write()
+            self._write(1, "def from_exception(self, error):")
+            self._write(2, "# type: (Exception) -> Union[Error, None]")
+            self._write(2, "if isinstance(error, Error):")
+            self._write(3, "return error")
+            self._write(2, "elif isinstance(error, grpc.RpcError):")
+            self._write(3, "err = self._deserialize_error(error.details())")
+            self._write(3, "if err is not None:")
+            self._write(4, "return err")
+            self._write(3, "err = self.error()")
+            self._write(3, "err.code = error.code().value[0]")
+            self._write(3, "err.errors = [error.details()]")
+            self._write(3, "return err")
+            self._write(2, "elif isinstance(error, Exception):")
+            self._write(3, "if len(error.args) != 1:")
+            self._write(4, "return None")
+            self._write(3, "if isinstance(error.args[0], Error):")
+            self._write(4, "return error.args[0]")
+            self._write(3, "elif isinstance(error.args[0], str):")
+            self._write(4, "return self._deserialize_error(error.args[0])")
+
             for method in methods:
                 print("generating method %s" % method["name"])
                 self._write()
@@ -762,6 +947,16 @@ class Generator:
         if not self._generate_version_api:
             return
 
+        # adding functionality for adding component info
+        self._write(
+            1,
+            "def set_component_info(self, client_name, client_version, server_name):",
+        )
+        self._write(2, "self._client_name = client_name")
+        self._write(2, "self._client_app_ver = client_version")
+        self._write(2, "self._server_name = server_name")
+        self._write()
+
         self._write(
             indent=1,
             line="""def _check_client_server_version_compatibility(
@@ -794,9 +989,11 @@ class Generator:
             raise Exception(err)
 
     def get_local_version(self):
+        log.info("Local Version is " + str(self._version_meta))
         return self._version_meta
 
     def get_remote_version(self):
+        log.info("Remote Version is " + str(self.get_version()))
         return self.get_version()
 
     def check_version_compatibility(self):
@@ -818,10 +1015,16 @@ class Generator:
                 local.api_spec_version, remote.api_spec_version, "API spec"
             )
         except Exception as e:
-            msg = "client SDK version '{}' is not compatible with server SDK version '{}'".format(
-                local.sdk_version, remote.sdk_version
-            )
-            return Exception("{}: {}".format(msg, str(e))), None
+            if self._client_name is not None:
+                msg = "{} {} is not compatible with {} {}".format(
+                    self._client_name, self._client_app_ver, self._server_name, remote.app_version
+                )
+                return Exception(msg), None
+            else:
+                msg = "client SDK version '{}' is not compatible with server SDK version '{}'".format(
+                    local.sdk_version, remote.sdk_version
+                )
+                return Exception("{}: {}".format(msg, str(e))), None
 
         return None, None
 
@@ -905,6 +1108,18 @@ class Generator:
         schema_object = self._get_object_from_ref(ref)
         ref_name = ref.split("/")[-1]
         class_name = ref_name.replace(".", "")
+        if "type" in schema_object and schema_object["type"] == "array":
+            schema_object["properties"] = {
+                ref_name.lower().replace(".", "_")
+                + "_"
+                + "list": {
+                    "type": "array",
+                    "items": {"$ref": schema_object["items"]["$ref"]},
+                    "x-field-uid": 1,
+                }
+            }
+            del schema_object["type"]
+            del schema_object["items"]
         if class_name in self._generated_classes:
             return
         self._generated_classes.append(class_name)
@@ -1129,7 +1344,6 @@ class Generator:
             choice_names = self._get_choice_names(schema_object)
             excluded_property_names = []
             for choice_name in choice_names:
-
                 # this code is to allow choices with no properties
                 if choice_name not in schema_object["properties"]:
                     excluded_property_names.append(choice_name)
@@ -1272,10 +1486,15 @@ class Generator:
                 for property, item in yobject["properties"].items():
                     if property == "choice":
                         continue
+                    ref = (
+                        item["items"]["$ref"]
+                        if "items" in item
+                        else item["$ref"]
+                    )
                     self._write_factory_method(
                         contained_class_name,
                         property,
-                        item["$ref"],
+                        ref,
                         True,
                         True,
                     )
@@ -1652,6 +1871,7 @@ class Generator:
 
     def _get_openapi_types(self, yobject):
         types = []
+        # TODO: revisit this portion for data type refactor
         if "properties" in yobject:
             for name in yobject["properties"]:
                 yproperty = yobject["properties"][name]
@@ -1659,9 +1879,10 @@ class Generator:
                 pt = {}
                 if "type" in yproperty:
                     pt.update({"type": self._get_data_types(yproperty)})
-                    pt.update(
-                        {"enum": yproperty["enum"]}
-                    ) if "enum" in yproperty else None
+                    if "enum" in yproperty:
+                        pt.update({"enum": yproperty["enum"]})
+                    elif "items" in yproperty and "enum" in yproperty["items"]:
+                        pt.update({"enum": yproperty["items"]["enum"]})
                     pt.update(
                         {"format": "'%s'" % yproperty["format"]}
                     ) if "format" in yproperty else None
@@ -1686,14 +1907,36 @@ class Generator:
                                 % yproperty["items"]["format"]
                             }
                         )
-                min_max = yproperty.get("maximum", yproperty.get("minimum", 0))
+
+                if "items" in yproperty:
+                    item_prop = yproperty["items"]
+                    if item_prop.get("minimum") is not None:
+                        yproperty["minimum"] = item_prop.get("minimum")
+                    if item_prop.get("maximum") is not None:
+                        yproperty["maximum"] = item_prop.get("maximum")
+                    if item_prop.get("minLength") is not None:
+                        yproperty["minLength"] = item_prop.get("minLength")
+                    if item_prop.get("maxLength") is not None:
+                        yproperty["maxLength"] = item_prop.get("maxLength")
+                    if item_prop.get("pattern") is not None:
+                        yproperty["pattern"] = item_prop.get("pattern")
+
                 key = (
                     "itemformat"
                     if pt.get("itemtype") is not None
                     else "format"
                 )
-                if min_max > 2147483647:
-                    pt.update({key: r"'int64'"})
+
+                if len(ref) == 0 and (
+                    pt.get("type") == "int" or pt.get("itemtype") == "int"
+                ):
+                    fmt = pt.get(key)
+                    fmt = fmt.replace("'", "") if fmt is not None else fmt
+                    final_fmt = type_limits._get_integer_format(
+                        fmt, yproperty.get("minimum"), yproperty.get("maximum")
+                    )
+                    pt.update({key: "'%s'" % final_fmt})
+
                 if len(ref) == 0 and "minimum" in yproperty:
                     pt.update({"minimum": yproperty["minimum"]})
                 if len(ref) == 0 and "maximum" in yproperty:
@@ -1702,6 +1945,8 @@ class Generator:
                     pt.update({"minLength": yproperty["minLength"]})
                 if len(ref) == 0 and "maxLength" in yproperty:
                     pt.update({"maxLength": yproperty["maxLength"]})
+                if len(ref) == 0 and "pattern" in yproperty:
+                    pt.update({"pattern": "r'%s'" % yproperty["pattern"]})
                 if len(pt) > 0:
                     types.append((name, pt))
                 # TODO: restore behavior
@@ -1898,3 +2143,108 @@ class Generator:
 
     def _write(self, indent=0, line=""):
         self._fid.write("    " * indent + line + "\n")
+
+    def _camelcase_to_snakecase(self, value):
+        word = ""
+        if value is None:
+            return word
+
+        insert_underscore = False
+
+        for c in value:
+            if c.isupper() or c.isdigit():
+                if insert_underscore:
+                    word += "_" + c.lower()
+                    insert_underscore = False
+                else:
+                    word += c.lower()
+            else:
+                word += c
+                insert_underscore = True
+
+        return word
+
+    def _get_ref_from_response(self, schema_obj, response_property):
+        ref = None
+        response_name = None
+        response_type = None
+        class_name = None
+        if len(schema_obj) == 0:
+            (
+                response_name,
+                _,
+                class_name,
+                ref,
+            ) = self._get_object_property_class_names(response_property)
+            proto_name = self._camelcase_to_snakecase(class_name)
+            if response_name is not None:
+                response = self._get_parser('$.."$ref"').find(
+                    self._get_object_from_ref(ref)
+                )
+                if len(response) > 0:
+                    (
+                        _,
+                        response_type,
+                        class_name,
+                        ref,
+                    ) = self._get_object_property_class_names(
+                        response[0].value
+                    )
+                    proto_name = self._camelcase_to_snakecase(class_name)
+        else:
+            (
+                _,
+                response_type,
+                class_name,
+                ref,
+            ) = self._get_object_property_class_names(schema_obj[0].value)
+            proto_name = self._camelcase_to_snakecase(class_name)
+
+        return response_name, response_type, class_name, ref, proto_name
+
+    def _add_streaming_code(self, rpc_method, return_byte, line_indent):
+        skip_rpc_conversion = False
+        obj = "req_obj" if rpc_method.request_class else "empty"
+        if rpc_method.stream_type != "":
+            self._write(
+                line_indent,
+                "if self.enable_grpc_streaming {}:".format(
+                    "and len(pb_str) > self._chunk_size"
+                    if rpc_method.stream_type == "client"
+                    else ""
+                ),
+            )
+            line_indent += 1
+            if rpc_method.stream_type == "client":
+                self._write(
+                    line_indent,
+                    "stream_req = self._client_stream(stub, pb_str)",
+                )
+                self._write(
+                    line_indent,
+                    "res_obj = stub.%s(stream_req, timeout=self._request_timeout)"
+                    % rpc_method.stream_operation_name,
+                )
+            elif rpc_method.stream_type == "server":
+                self._write(
+                    line_indent,
+                    "res = stub.%s(%s, timeout=self._request_timeout)"
+                    % (rpc_method.stream_operation_name, obj),
+                )
+                self._write(
+                    line_indent, "data = self._server_stream(stub, res)"
+                )
+                if return_byte:
+                    self._write(line_indent, "return io.BytesIO(data)")
+                    skip_rpc_conversion = True
+                elif rpc_method.good_response_type:
+                    self._write(
+                        line_indent,
+                        "res_obj = pb2.%s()" % rpc_method.response_class,
+                    )
+                    self._write(line_indent, "res_obj.ParseFromString(data)")
+                else:
+                    self._write(line_indent, "return data.decode()")
+                    skip_rpc_conversion = True
+            self._write(line_indent - 1, "else:")
+        return line_indent, skip_rpc_conversion
