@@ -1,6 +1,6 @@
 import subprocess
 import os
-from .openapiartplugin import OpenApiArtPlugin
+from .openapiartplugin import OpenApiArtPlugin, type_limits
 
 
 class OpenApiArtProtobuf(OpenApiArtPlugin):
@@ -30,6 +30,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             "responses"
         ].items():
             self._write_msg(name, response_object)
+        self._write_data_msg()
         for _, path_object in self._openapi["paths"].items():
             self._write_request_msg(path_object)
             self._write_response_msg(path_object)
@@ -76,6 +77,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     )
                     > 0
                 )
+                operation.stream_type = path_item_object.get("x-stream")
                 self._operations[operation_id] = operation
             return self._operations[operation_id]
         return None
@@ -92,6 +94,15 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                 operation.request = "{}Request".format(operation.rpc)
                 self._write()
                 self._write("message {} {{".format(operation.request))
+                if (
+                    len(
+                        self._get_parser(
+                            '$..requestBody.."application/octet-stream"'
+                        ).find(path_item_object)
+                    )
+                    > 0
+                ):
+                    self._write("bytes request_bytes = 1;", indent=1)
                 for ref in self._get_parser('$..requestBody.."$ref"').find(
                     path_item_object
                 ):
@@ -168,10 +179,14 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     response_fields.append(response_field)
             self._write("message {} {{".format(operation.response))
             for response_field in response_fields:
+                if response_field.type == "Error":
+                    continue
                 self._write(
-                    "optional {} {} = {};".format(
+                    "{} {} = {};".format(
                         response_field.type,
-                        response_field.name,
+                        self._lowercase(response_field.type)
+                        if response_field.type != "bytes"
+                        else "response_bytes",
                         response_field.field_uid,
                     ),
                     indent=1,
@@ -262,7 +277,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     max = outer_max
                 if outer_format is not None:
                     type_format = outer_format
-                return _get_integer_format(type_format, min, max)
+                return type_limits._get_integer_format(type_format, min, max)
             if type == "number":
                 if "format" in openapi_object:
                     if (
@@ -290,11 +305,19 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     outer_max=max,
                 )
 
+                if item_type.startswith("object_"):
+                    prefix_len = len("object_")
+                    item_type = item_type[prefix_len:]
+
                 if item_type == "integer":
-                    item_type = _get_integer_format(type_format, min, max)
+                    item_type = type_limits._get_integer_format(
+                        type_format, min, max
+                    )
                 return "repeated " + item_type
         elif "$ref" in openapi_object:
-            return openapi_object["$ref"].split("/")[-1].replace(".", "")
+            return "object_" + openapi_object["$ref"].split("/")[-1].replace(
+                ".", ""
+            )
 
     def _camelcase(self, value):
         camel_case = ""
@@ -345,7 +368,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             self._write("{} = {};".format("unspecified", 0), indent=3)
         for key, value in enums.items():
             field_uid = value["x-field-uid"]
-            self._write("{} = {};".format(key.lower(), field_uid), indent=3)
+            self._write("{} = {};".format(key, field_uid), indent=3)
         self._write("}", indent=2)
         self._write("}", indent=1)
 
@@ -377,6 +400,18 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             print("Failed writing response {}: {}".format(msg_name, err))
 
     def _write_msg_fields(self, name, schema_object):
+        if "type" in schema_object and schema_object["type"] == "array":
+            schema_object["properties"] = {
+                name.lower().replace(".", "_")
+                + "_"
+                + "list": {
+                    "type": "array",
+                    "items": {"$ref": schema_object["items"]["$ref"]},
+                    "x-field-uid": 1,
+                }
+            }
+            del schema_object["type"]
+            del schema_object["items"]
         if "properties" not in schema_object:
             return
         for property_name, property_object in schema_object[
@@ -394,19 +429,21 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     default = "{}.{}".format(
                         property_type.split(" ")[-1], default.lower()
                     )
-            if (
-                "required" in schema_object
-                and property_name in schema_object["required"]
-                or property_type.startswith("repeated")
-            ):
+            if property_type.startswith(
+                "repeated"
+            ) or property_type.startswith("object_"):
                 optional = ""
+                if property_type.startswith("object_"):
+                    prefix_len = len("object_")
+                    property_type = property_type[prefix_len:]
             else:
                 optional = "optional "
             desc = self._get_description(property_object)
             if default is not None:
                 desc += "\ndefault = {}".format(default)
             if (
-                optional == ""
+                "required" in schema_object
+                and property_name in schema_object["required"]
                 and property_type.startswith("repeated") is not True
             ):
                 desc += "\nrequired = true"
@@ -422,9 +459,19 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
     def _write_service(self):
         self._write()
         paths_object = self._openapi["paths"]
+        self._write(self._justify_desc(self._get_description(paths_object)))
+        self._write("//")
         self._write(
-            self._justify_desc(self._get_description(paths_object), indent=1)
+            "// For all RPCs defined in this service, API Server SHOULD provide JSON"
         )
+        self._write(
+            "// representation of `Error` message as an error string upon failure, ensuring"
+        )
+        self._write(
+            "// name of enum constants (instead of value) for `kind` property is present"
+        )
+        self._write("// in the representation")
+
         self._write("service {name} {{".format(name=self.proto_service_name))
         self._write()
         for url, path_object in self._openapi["paths"].items():
@@ -446,75 +493,35 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             operation.rpc, operation.request, "", operation.response
         )
         self._write(line, indent=1)
+        # additional code to add a stream rpc under the hood
+        if operation.stream_type and operation.stream_type == "client":
+            self._write(
+                "// streaming version of the rpc {}".format(operation.rpc),
+                indent=1,
+            )
+            print("writing rpc {}".format("stream" + operation.rpc))
+            line = "rpc {}(stream Data) returns ({});".format(
+                "stream" + operation.rpc, operation.response
+            )
+            self._write(line, indent=1)
+        elif operation.stream_type and operation.stream_type == "server":
+            self._write(
+                "// streaming version of the rpc {}".format(operation.rpc),
+                indent=1,
+            )
+            print("writing rpc {}".format("stream" + operation.rpc))
+            line = "rpc {}({}) returns (stream Data);".format(
+                "stream" + operation.rpc, operation.request
+            )
+            self._write(line, indent=1)
 
-
-def _min_max_in_range(format, min, max):
-    if min is None or max is None:
-        return True
-    elif min is None or max is None:
-        if min is None:
-            min = max
-        if max is None:
-            max = min
-
-    if min > max:
-        return False
-
-    if format == "uint32":
-        return min >= 0 and max <= 4294967295
-    elif format == "uint64":
-        return min >= 0 and max <= 18446744073709551615
-    elif format == "int32":
-        return min >= -2147483648 and max <= 2147483647
-    elif format == "int64":
-        return min >= -9223372036854775808 and max <= 9223372036854775807
-    else:
-        raise Exception("unsupported integer format: {}".format(format))
-
-
-def _format_from_range(min, max):
-    if min is None and max is None:
-        return "int32"
-    elif min is None or max is None:
-        if min is None:
-            min = max
-        if max is None:
-            max = min
-
-    if max < min:
-        raise Exception("min %d cannot be less than max %d", min, max)
-
-    # TODO: this logic needs to be replaced with the one commented out below
-    if min > 2147483647 or max > 2147483647:
-        return "int64"
-    else:
-        return "int32"
-    # TODO: following snippet is more accurate but introduces breaking
-    # changes and hence commented out
-    # if self._min_max_in_range("uint32", min, max):
-    #     return "uint32"
-    # if self._min_max_in_range("uint64", min, max):
-    #     return "uint64"
-    # if self._min_max_in_range("int32", min, max):
-    #     return "int32"
-    # if self._min_max_in_range("int64", min, max):
-    #     return "int64"
-
-
-def _get_integer_format(type_format, min, max):
-    valid_formats = ["int32", "int64", "uint32", "uint64"]
-    if type_format is not None:
-        if type_format in valid_formats:
-            if not _min_max_in_range(type_format, min, max):
-                raise Exception(
-                    "format {} is not compatible with [min,max] [{},{}]".format(
-                        type_format, min, max
-                    )
-                )
-            return type_format
-        raise Exception(
-            "unsupported format %s, supported formats are: %s",
-            type_format,
-            valid_formats,
-        )
-    return _format_from_range(min, max)
+    def _write_data_msg(self):
+        # This function is a must have a message to support grpc streaming.
+        # Its not mentioned in the model so needs to be handled by openapiart under the hood
+        print("writing msg Data for streaming")
+        self._write()
+        self._write("// Data that needs to be streamed")
+        self._write("message Data {")
+        self._write("uint64 chunk_size = 1;", indent=1)
+        self._write("bytes datum = 2;", indent=1)
+        self._write("}")
