@@ -18,13 +18,23 @@ app.HOST = "0.0.0.0"
 def set_config():
     config = app.PACKAGE.Api().prefix_config()
     config.deserialize(request.data.decode("utf-8"))
-    test = config.h
-    if test is not None and isinstance(test, bool) is False:
+    test = config.c
+    if test is not None and test == 500:
         return Response(
-            status=590,
+            status=500,
             response=json.dumps({"detail": "invalid data type"}),
             headers={"Content-Type": "application/json"},
         )
+    elif test is not None and test == 400:
+        err = app.PACKAGE.Api().error()
+        err.code = 400
+        err.kind = "validation"
+        err.errors = ["err for validation"]
+        return Response(
+            status=400,
+            response=err.serialize(),
+            headers={"Content-Type": "application/json"},
+        )
     else:
         app.CONFIG = config
         return Response(status=200)
@@ -38,6 +48,14 @@ def update_configuration():
     return Response(status=200)
 
 
+@app.route("/api/config/append", methods=["PATCH"])
+def append_config():
+    config = app.PACKAGE.Api().warning_details()
+    config.warnings = ["w1", "w2"]
+    serialized_config = config.serialize()
+    return Response(serialized_config, mimetype="application/json", status=200)
+
+
 @app.route("/api/config", methods=["GET"])
 def get_config():
     app.CONFIG.a = "asdf"
@@ -58,6 +76,17 @@ def get_version():
     )
 
 
+@app.route("/api/file/upload", methods=["POST"])
+def upload_config():
+    print("-------------------------------------------------")
+    print(request.data, type(request.data))
+    print("-------------------------------------------------")
+    config = app.PACKAGE.Api().warning_details()
+    config.warnings = ["w1", "w2"]
+    serialized_config = config.serialize()
+    return Response(serialized_config, mimetype="application/json", status=200)
+
+
 @app.after_request
 def after_request(resp):
     print(request.method, request.url, " -> ", resp.status)
