@@ -2,7 +2,6 @@ import pytest
 
 
 def test_warnings_with_api(api, capfd):
-
     u_conf = api.update_config()
     u_conf.g.gobject(g_a="abcd")
 
@@ -11,7 +10,7 @@ def test_warnings_with_api(api, capfd):
     print(out, err)
     assert (
         "[WARNING]: update_configuration api is deprecated, please use post instead"
-        in out
+        in err
     )
     assert api.__warnings__ != []
     assert len(api.__warnings__) == 1
@@ -31,46 +30,46 @@ def test_warning_for_primitive_attr(api, capsys):
 
     conf.space_1 = 56
     conf.str_len = "1245"
-    conf.hex_slice = ["str1", "str2"]
+    conf.hex_slice = ["0xff", "0xab"]
     s_obj = conf.serialize(conf.DICT)
 
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "[WARNING]: space_1 property in schema PrefixConfig is deprecated, Information TBD"
-        in out
+        in err
     )
     assert (
         "[WARNING]: a property in schema PrefixConfig is under_review, Information TBD"
-        in out
+        in err
     )
     assert (
         "[WARNING]: str_len property in schema PrefixConfig is under_review, Information TBD"
-        in out
+        in err
     )
     assert (
         "[WARNING]: hex_slice property in schema PrefixConfig is under_review, Information TBD"
-        in out
+        in err
     )
 
     conf.deserialize(s_obj)
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "[WARNING]: space_1 property in schema PrefixConfig is deprecated, Information TBD"
-        in out
+        in err
     )
     assert (
         "[WARNING]: a property in schema PrefixConfig is under_review, Information TBD"
-        in out
+        in err
     )
     assert (
         "[WARNING]: str_len property in schema PrefixConfig is under_review, Information TBD"
-        in out
+        in err
     )
     assert (
         "[WARNING]: hex_slice property in schema PrefixConfig is under_review, Information TBD"
-        in out
+        in err
     )
 
 
@@ -88,18 +87,18 @@ def test_warnings_for_non_primitive_attr(api, capsys):
     s_obj = conf.serialize(conf.DICT)
     out, err = capsys.readouterr()
 
-    assert err == ""
+    assert out == ""
     assert (
         "e property in schema PrefixConfig is deprecated, Information TBD"
-        in out
+        in err
     )
 
     conf.deserialize(s_obj)
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "e property in schema PrefixConfig is deprecated, Information TBD"
-        in out
+        in err
     )
 
 
@@ -118,9 +117,9 @@ def test_warnings_for_iter_items(api, capsys):
     s_obj = conf.serialize(conf.DICT)
     out, err = capsys.readouterr()
 
-    assert err == ""
+    assert out == ""
     assert (
-        out.count(
+        err.count(
             "[WARNING]: g_c property in schema GObject is deprecated, Information TBD"
         )
         == 3
@@ -128,9 +127,9 @@ def test_warnings_for_iter_items(api, capsys):
 
     conf.deserialize(s_obj)
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
-        out.count(
+        err.count(
             "[WARNING]: g_c property in schema GObject is deprecated, Information TBD"
         )
         == 3
@@ -149,18 +148,18 @@ def test_warnings_for_x_enmu_attr(api, capsys):
     s_obj = conf.serialize(conf.DICT)
 
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "[WARNING]: STATUS_404 enum in property response is deprecated, new code will be coming soon"
-        in out
+        in err
     )
 
     conf.deserialize(s_obj)
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "[WARNING]: STATUS_404 enum in property response is deprecated, new code will be coming soon"
-        in out
+        in err
     )
 
 
@@ -177,18 +176,18 @@ def test_warnings_for_choice_attr(api, capsys):
     s_obj = conf.serialize(conf.DICT)
 
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "[WARNING]: J_B enum in property choice is deprecated, use j_a instead"
-        in out
+        in err
     )
 
     conf.deserialize(s_obj)
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
         "[WARNING]: J_B enum in property choice is deprecated, use j_a instead"
-        in out
+        in err
     )
 
 
@@ -199,21 +198,21 @@ def test_warnings_for_schema(api, capsys):
     s_obj = conf.serialize(conf.DICT)
 
     out, err = capsys.readouterr()
-    assert err == ""
+    assert out == ""
     assert (
-        out.count(
+        err.count(
             "[WARNING]: UpdateConfig is under_review, the whole schema is being reviewed"
         )
         == 1
     )
     assert (
-        out.count(
+        err.count(
             "[WARNING]: GObject is deprecated, new schema Jobject to be used"
         )
         == 2
     )
     assert (
-        out.count(
+        err.count(
             "[WARNING]: g_c property in schema GObject is deprecated, Information TBD"
         )
         == 2
@@ -222,21 +221,21 @@ def test_warnings_for_schema(api, capsys):
     conf.deserialize(s_obj)
     out, err = capsys.readouterr()
 
-    assert err == ""
+    assert out == ""
     assert (
-        out.count(
+        err.count(
             "[WARNING]: UpdateConfig is under_review, the whole schema is being reviewed"
         )
         == 1
     )
     assert (
-        out.count(
+        err.count(
             "[WARNING]: GObject is deprecated, new schema Jobject to be used"
         )
         == 2
     )
     assert (
-        out.count(
+        err.count(
             "[WARNING]: g_c property in schema GObject is deprecated, Information TBD"
         )
         == 2
